
clear; clc;
addpath('MISC_fn');
addpath('Ucsd_garch');
addpath('MFEToolbox');
%% loading in real data!
load('rtrvbv.mat');

%take a subset - 1750 obs with a 250 obs hold out for prediction
TT=1750;
m=250;
T=TT-m;
yout=RT(end-m+1:end);
BVout=BV(end-m+1:end);
RVout=RV(end-m+1:end);
y=RT(end-TT+1:end-m);
RV=RV(end-TT+1:end-m);
BV=BV(end-TT+1:end-m);
lnBV=log(BV);
lnRV=log(RV);
JV=RV-BV;
JV(JV<=0)=0; %take max(RV-BV,0);
sRT=y./sqrt(BV);
%% estimate the GARCH model (as in SSM example)
tic;
%Change the input options here for alternative specifications of GARCH
%models.
[parameters, LL, ht, robustSE, VCV, dat_score, diagnostics] = tarch(y, 1, 0, 1, 'NORMAL',2);

X=[ones(T,1),log(ht)];
b=regress(lnBV,X);
sig=sqrt(var(lnBV-sum(repmat(b',T,1).*X,2)));
lnBVstats=[b',sig,skewness(lnBV),kurtosis(lnBV)];

sJV=sign(y).*sqrt(JV);
JVstats=[mean(sJV),var(JV),corr(JV(1:end-1),JV(2:end))];
%append further stats on BV/JV etc.
dat_stat=[mean(dat_score),lnBVstats,JVstats];

%% setting up ABC stuffs
%number of ABC draws per run - a function of T
eps=50/(T.^(3/2));
P=500;
reps=round(P/eps);
%storage vector for the summary stats
sim_stat=zeros(reps,length(dat_stat));

%% simulating from the prior
mup=-1+(1-(-1))*rand(reps,1);
sigp=0.5+(3-0.5)*rand(reps,1);
del0=0.001+(0.3-0.001)*rand(reps,1);
ap=0.001+(0.2-0.001)*rand(reps,1);
bp=0.5+(1-ap-0.5).*rand(reps,1);
psi0=-0.5+(0.5-(-0.5))*rand(reps,1);
psi1=0.5+(1.5-0.5)*rand(reps,1);
sigbv=0.001+(1-0.001)*rand(reps,1);
omgv=-1+(1-(-1))*rand(reps,1);
rho=0.5+(0.99-0.5)*rand(reps,1);
sigv=0.001+(0.3-0.001)*rand(reps,1);
alpha=1.5+(2-1.5)*rand(reps,1);

parfor i=1:reps
   [ sim_r, sim_lnBV, sim_x, sim_Jp, sim_delt ] = gen_RTBVModel( T, [mup(i),sigp(i),del0(i),ap(i),bp(i),psi0(i),psi1(i),sigbv(i),omgv(i),rho(i),sigv(i),alpha(i)] ); 
   sim_lnRV=log(exp(sim_lnBV)+sim_Jp.^2);
   sim_JV=sim_Jp.^2;
   sim_sJV=sign(sim_r).*sqrt(sim_JV);
   [sim_score]=tarch_scores(sim_r, 1, 0, 1, 1, 2, parameters); %error_type=2 stands for StudentT
   
%    sim_lnBVstats=[mean(sim_lnBV), var(sim_lnBV), skewness(sim_lnBV), kurtosis(sim_lnBV),corr(sim_lnBV(1:end-1),sim_lnBV(2:end))];
    [sim_ht]=garch_h(sim_r,parameters);
    X=[ones(T,1),log(sim_ht)];
    b=regress(sim_lnBV,X);
    sig=sqrt(var(sim_lnBV-sum(repmat(b',T,1).*X,2)));
    sim_lnBVstats=[b',sig,skewness(lnBV),kurtosis(lnBV)];

   sim_JVstats=[mean(sim_sJV),var(sim_JV),corr(sim_JV(1:end-1),sim_JV(2:end))];
   
   stat=[mean(sim_score),sim_lnBVstats,sim_JVstats];
   sim_stat(i,:)=stat;
end

%trapping out NaN records
theta_draw=[mup,sigp,del0,ap,bp,psi0,psi1,sigbv,omgv,rho,sigv,alpha];
theta_draw(any(isnan(sim_stat), 2), :)=[];
sim_stat(any(isnan(sim_stat), 2), :)=[];


[sstat_dist]=sstatDist_Euclidean(sim_stat, dat_stat, length(sim_stat));
theta_dist=[theta_draw,sstat_dist];
theta_dist_sorted=sortrows(theta_dist,13);
theta_dist=theta_dist_sorted(1:P,1:12);

ABCtime=toc;

save('realdata_ABC_GARCHNormhReg.mat');

%%
figure;
subplot(2,2,1); plot((-1:2/99:1),ksdensity(theta_dist(:,1),(-1:2/99:1))); title('mup');
subplot(2,2,2); plot((0.5:2.5/99:3),ksdensity(theta_dist(:,2),(0.5:2.5/99:3))); title('sigp');
subplot(2,2,3); plot((0:.3/99:.3),ksdensity(theta_dist(:,3),(0:.3/99:.3))); title('del0');
subplot(2,2,4); plot((0:.2/99:.2),ksdensity(theta_dist(:,4),(0:.2/99:.2))); title('ap');

figure;
subplot(2,2,1); plot((0.5:0.5/99:1),ksdensity(theta_dist(:,5),(0.5:0.5/99:1))); title('bp');
subplot(2,2,2); plot((-0.5:1/99:0.5),ksdensity(theta_dist(:,6),(-0.5:1/99:0.5))); title('psi0');
subplot(2,2,3); plot((0.5:1/99:1.5),ksdensity(theta_dist(:,7),(0.5:1/99:1.5))); title('psi1');
subplot(2,2,4); plot((0:1/99:1),ksdensity(theta_dist(:,8),(0:1/99:1))); title('sigbv');

figure;
subplot(2,2,1); plot((-1:2/99:1),ksdensity(theta_dist(:,9),(-1:2/99:1))); title('omgv');
subplot(2,2,2); plot((0.5:0.49/99:0.99),ksdensity(theta_dist(:,10),(0.5:0.49/99:0.99))); title('rho');
subplot(2,2,3); plot((0:.3/99:.3),ksdensity(theta_dist(:,11),(0:.3/99:.3))); title('sigv');
subplot(2,2,4); plot((1.5:0.5/99:2),ksdensity(theta_dist(:,12),(1.5:0.5/99:2))); title('alpha');

%% regression adjustment for each column of draw

[~,r]=size(theta_draw);
theta_reg_sorted=zeros(P,r);

for i=1:r
    [fp_dist]= sstatDist_reg(sim_stat, dat_stat, theta_draw(:,i));
    tmp=sortrows([theta_draw(:,i),fp_dist],2);
    theta_reg_sorted(:,i)=tmp(1:P,1);
end
save('realdata_ABC_GARCHNormhReg.mat');


%% posterior statistics
MPM=mean(theta_dist);
q=[0.025 0.975];
HPD=quantile(theta_dist,q);