%% loop to obtain predictive distributions
clear; clc;
addpath('MISC_fn');
load('rtrvbv.mat');

%take a subset - 1750 obs with a 250 obs hold out for prediction
TT=1750;
m=250;
T=TT-m;
yout=RT(end-m+1:end);
BVout=BV(end-m+1:end);
RVout=RV(end-m+1:end);
y=RT(end-TT+1:end);
RV=RV(end-TT+1:end);
BV=BV(end-TT+1:end);
lnBV=log(BV);

%load in posterior draws
GARCHN=load('realdata_ABC_GARCHNormhReg.mat','theta_dist');
GARCHT=load('realdata_ABC_GARCHhReg.mat','theta_dist');
TGARCHT=load('realdata_ABC_TGARCHhReg.mat','theta_dist');
RGARCH=load('realdata_ABC_RealizedGARCH.mat','theta_dist');

P=length(GARCHT.theta_dist);
g=100;
ygrid=linspace(-5,5,g);
lnBVgrid=linspace(-6,6,g);
GARCHN_predy=zeros(m,g,P);
GARCHN_predbv=zeros(m,g,P);
GARCHT_predy=zeros(m,g,P);
GARCHT_predbv=zeros(m,g,P);
TGARCHT_predy=zeros(m,g,P);
TGARCHT_predbv=zeros(m,g,P);
RGARCH_predy=zeros(m,g,P);
RGARCH_predbv=zeros(m,g,P);

parfor i=1:P
   [ GARCHN_predy(:,:,i), GARCHN_predbv(:,:,i) ] = predY_BPF_SVaStabJump( ygrid, lnBVgrid, m, y, lnBV, GARCHN.theta_dist(i,:) ); 

   [ GARCHT_predy(:,:,i), GARCHT_predbv(:,:,i) ] = predY_BPF_SVaStabJump( ygrid, lnBVgrid, m, y, lnBV, GARCHT.theta_dist(i,:) ); 

   [ TGARCHT_predy(:,:,i), TGARCHT_predbv(:,:,i) ] = predY_BPF_SVaStabJump( ygrid, lnBVgrid, m, y, lnBV, TGARCHT.theta_dist(i,:) ); 

   [ RGARCH_predy(:,:,i), RGARCH_predbv(:,:,i) ] = predY_BPF_SVaStabJump( ygrid, lnBVgrid, m, y, lnBV, RGARCH.theta_dist(i,:) ); 
end

GARCHN_postpredy=mean(GARCHN_predy,3);
GARCHN_postpredbv=mean(GARCHN_predbv,3);
GARCHT_postpredy=mean(GARCHT_predy,3);
GARCHT_postpredbv=mean(GARCHT_predbv,3);
TGARCHT_postpredy=mean(TGARCHT_predy,3);
TGARCHT_postpredbv=mean(TGARCHT_predbv,3);
RGARCH_postpredy=mean(RGARCH_predy,3);
RGARCH_postpredbv=mean(RGARCH_predbv,3);

save('predictive_results.mat');

%% filtering out NaNs from GARCHN and RGARCH predictions
tmppbv=zeros(m,g);
tmppy=zeros(m,g);
count1=0;
count2=0;
for i=1:P
    if(sum(sum(isnan(GARCHN_predy(:,:,i))))==0)
       count1=count1+1;
       tmppy=tmppy+GARCHN_predy(:,:,i); 
    end
    
    if(sum(sum(isnan(GARCHN_predbv(:,:,i))))==0)
       count2=count2+1;
       tmppbv=tmppbv+GARCHN_predbv(:,:,i); 
    end
end

GARCHN_postpredy=tmppy/count1;
GARCHN_postpredbv=tmppbv/count2;

tmppbv=zeros(m,g);
tmppy=zeros(m,g);
count1=0;
count2=0;
for i=1:P
    if(sum(sum(isnan(RGARCH_predy(:,:,i))))==0)
       count1=count1+1;
       tmppy=tmppy+RGARCH_predy(:,:,i); 
    end
    
    if(sum(sum(isnan(RGARCH_predbv(:,:,i))))==0)
       count2=count2+1;
       tmppbv=tmppbv+RGARCH_predbv(:,:,i); 
    end
end

RGARCH_postpredy=tmppy/count1;
RGARCH_postpredbv=tmppbv/count2;
save('predictive_results.mat');

%% example plots

figure;
subplot(2,1,1); plot(ygrid,[GARCHN_postpredy(1,:)' GARCHT_postpredy(1,:)' TGARCHT_postpredy(1,:)' RGARCH_postpredy(1,:)']);
hold on;
line([yout(1) yout(1)],get(gca,'YLim'),'Color',[1 0 0]);
title('Posterior Predictive Return Distribution 11-Feb-2016');
legend('GARCH-Normal', 'GARCH-T', 'TGARCH-T','RealizedGARCH','Location','northwest');

subplot(2,1,2); plot(lnBVgrid,[GARCHN_postpredbv(1,:)' GARCHT_postpredbv(1,:)' TGARCHT_postpredbv(1,:)' RGARCH_postpredbv(1,:)']);
hold on;
line([log(BVout(1)) log(BVout(1))],get(gca,'YLim'),'Color',[1 0 0]);
title('Posterior Predictive lnBV Distribution 11-Feb-2016');


figure;
subplot(2,1,1); plot(ygrid,[GARCHN_postpredy(100,:)' GARCHT_postpredy(100,:)' TGARCHT_postpredy(100,:)' RGARCH_postpredy(100,:)']);
hold on;
line([yout(100) yout(100)],get(gca,'YLim'),'Color',[1 0 0]);
title('Posterior Predictive Return Distribution 5-Jul-2016');
legend('GARCH-Normal', 'GARCH-T', 'TGARCH-T','RealizedGARCH','Location','northwest');

subplot(2,1,2); plot(lnBVgrid,[GARCHN_postpredbv(100,:)' GARCHT_postpredbv(100,:)' TGARCHT_postpredbv(100,:)' RGARCH_postpredbv(100,:)']);
hold on;
line([log(BVout(100)) log(BVout(100))],get(gca,'YLim'),'Color',[1 0 0]);
title('Posterior Predictive lnBV Distribution 5-Jul-2016');


figure;
subplot(2,1,1); plot(ygrid,[GARCHN_postpredy(end,:)' GARCHT_postpredy(end,:)' TGARCHT_postpredy(end,:)' RGARCH_postpredy(end,:)']);
hold on;
line([yout(end) yout(end)],get(gca,'YLim'),'Color',[1 0 0]);
title('Posterior Predictive Return Distribution 7-Feb-2017');
legend('GARCH-Normal', 'GARCH-T', 'TGARCH-T','RealizedGARCH','Location','northwest');

subplot(2,1,2); plot(lnBVgrid,[GARCHN_postpredbv(end,:)' GARCHT_postpredbv(end,:)' TGARCHT_postpredbv(end,:)' RGARCH_postpredbv(end,:)']);
hold on;
line([log(BVout(end)) log(BVout(end))],get(gca,'YLim'),'Color',[1 0 0]);
title('Posterior Predictive lnBV Distribution 7-Feb-2017');

%% compute the predictive scores across the 250 days

y_logscore=zeros(m,4);
y_QS=zeros(m,4);
y_CRPS=zeros(m,4);

bv_logscore=zeros(m,4);
bv_QS=zeros(m,4);
bv_CRPS=zeros(m,4);

for i=1:m
    [y_logscore(i,1), y_QS(i,1), y_CRPS(i,1)]=calcPredScores(ygrid,GARCHN_postpredy(i,:),yout(i),ygrid(1),ygrid(end));
    [y_logscore(i,2), y_QS(i,2), y_CRPS(i,2)]=calcPredScores(ygrid,GARCHT_postpredy(i,:),yout(i),ygrid(1),ygrid(end));
    [y_logscore(i,3), y_QS(i,3), y_CRPS(i,3)]=calcPredScores(ygrid,TGARCHT_postpredy(i,:),yout(i),ygrid(1),ygrid(end));
    [y_logscore(i,4), y_QS(i,4), y_CRPS(i,4)]=calcPredScores(ygrid,RGARCH_postpredy(i,:),yout(i),ygrid(1),ygrid(end));
    
    [bv_logscore(i,1), bv_QS(i,1), bv_CRPS(i,1)]=calcPredScores(lnBVgrid,GARCHN_postpredbv(i,:),yout(i),lnBVgrid(1),lnBVgrid(end));
    [bv_logscore(i,2), bv_QS(i,2), bv_CRPS(i,2)]=calcPredScores(lnBVgrid,GARCHT_postpredbv(i,:),yout(i),lnBVgrid(1),lnBVgrid(end));
    [bv_logscore(i,3), bv_QS(i,3), bv_CRPS(i,3)]=calcPredScores(lnBVgrid,TGARCHT_postpredbv(i,:),yout(i),lnBVgrid(1),lnBVgrid(end));
    [bv_logscore(i,4), bv_QS(i,4), bv_CRPS(i,4)]=calcPredScores(lnBVgrid,RGARCH_postpredbv(i,:),yout(i),lnBVgrid(1),lnBVgrid(end));
end

y_lsav=mean(y_logscore);
y_QSav=mean(y_QS);
y_CRPSav=mean(y_CRPS);

bv_lsav=mean(bv_logscore);
bv_QSav=mean(bv_QS);
bv_CRPSav=mean(bv_CRPS);
save('predictive_results.mat');

%% plots of posterior densities
grid=100;
mupgrid=linspace(-1,1,grid);
sigpgrid=linspace(0.5,3,grid);
del0grid=linspace(0.001,0.3,grid);
apgrid=linspace(0.001,0.2,grid);
bpgrid=linspace(0.5,0.99,grid);
psi0grid=linspace(-0.5,0.5,grid);
psi1grid=linspace(0.5,1.5,grid);
sigbvgrid=linspace(0.001,1,grid);
omgvgrid=linspace(-1,1,grid);
rhogrid=linspace(0.5,0.99,grid);
sigvgrid=linspace(0.001,0.3,grid);
alphagrid=linspace(1.5,2,grid);

GARCHN.mupKern=ksdensity(GARCHN.theta_dist(:,1),mupgrid);
GARCHN.sigpKern=ksdensity(GARCHN.theta_dist(:,2),sigpgrid);
GARCHN.del0Kern=ksdensity(GARCHN.theta_dist(:,3),del0grid);
GARCHN.apKern=ksdensity(GARCHN.theta_dist(:,4),apgrid);
GARCHN.bpKern=ksdensity(GARCHN.theta_dist(:,5),bpgrid);
GARCHN.psi0Kern=ksdensity(GARCHN.theta_dist(:,6),psi0grid);
GARCHN.psi1Kern=ksdensity(GARCHN.theta_dist(:,7),psi1grid);
GARCHN.sigbvKern=ksdensity(GARCHN.theta_dist(:,8),sigbvgrid);
GARCHN.omgvKern=ksdensity(GARCHN.theta_dist(:,9),omgvgrid);
GARCHN.rhoKern=ksdensity(GARCHN.theta_dist(:,10),rhogrid);
GARCHN.sigvKern=ksdensity(GARCHN.theta_dist(:,11),sigvgrid);
GARCHN.alphaKern=ksdensity(GARCHN.theta_dist(:,12),alphagrid);

GARCHT.mupKern=ksdensity(GARCHT.theta_dist(:,1),mupgrid);
GARCHT.sigpKern=ksdensity(GARCHT.theta_dist(:,2),sigpgrid);
GARCHT.del0Kern=ksdensity(GARCHT.theta_dist(:,3),del0grid);
GARCHT.apKern=ksdensity(GARCHT.theta_dist(:,4),apgrid);
GARCHT.bpKern=ksdensity(GARCHT.theta_dist(:,5),bpgrid);
GARCHT.psi0Kern=ksdensity(GARCHT.theta_dist(:,6),psi0grid);
GARCHT.psi1Kern=ksdensity(GARCHT.theta_dist(:,7),psi1grid);
GARCHT.sigbvKern=ksdensity(GARCHT.theta_dist(:,8),sigbvgrid);
GARCHT.omgvKern=ksdensity(GARCHT.theta_dist(:,9),omgvgrid);
GARCHT.rhoKern=ksdensity(GARCHT.theta_dist(:,10),rhogrid);
GARCHT.sigvKern=ksdensity(GARCHT.theta_dist(:,11),sigvgrid);
GARCHT.alphaKern=ksdensity(GARCHT.theta_dist(:,12),alphagrid);

TGARCHT.mupKern=ksdensity(TGARCHT.theta_dist(:,1),mupgrid);
TGARCHT.sigpKern=ksdensity(TGARCHT.theta_dist(:,2),sigpgrid);
TGARCHT.del0Kern=ksdensity(TGARCHT.theta_dist(:,3),del0grid);
TGARCHT.apKern=ksdensity(TGARCHT.theta_dist(:,4),apgrid);
TGARCHT.bpKern=ksdensity(TGARCHT.theta_dist(:,5),bpgrid);
TGARCHT.psi0Kern=ksdensity(TGARCHT.theta_dist(:,6),psi0grid);
TGARCHT.psi1Kern=ksdensity(TGARCHT.theta_dist(:,7),psi1grid);
TGARCHT.sigbvKern=ksdensity(TGARCHT.theta_dist(:,8),sigbvgrid);
TGARCHT.omgvKern=ksdensity(TGARCHT.theta_dist(:,9),omgvgrid);
TGARCHT.rhoKern=ksdensity(TGARCHT.theta_dist(:,10),rhogrid);
TGARCHT.sigvKern=ksdensity(TGARCHT.theta_dist(:,11),sigvgrid);
TGARCHT.alphaKern=ksdensity(TGARCHT.theta_dist(:,12),alphagrid);

RGARCH.mupKern=ksdensity(RGARCH.theta_dist(:,1),mupgrid);
RGARCH.sigpKern=ksdensity(RGARCH.theta_dist(:,2),sigpgrid);
RGARCH.del0Kern=ksdensity(RGARCH.theta_dist(:,3),del0grid);
RGARCH.apKern=ksdensity(RGARCH.theta_dist(:,4),apgrid);
RGARCH.bpKern=ksdensity(RGARCH.theta_dist(:,5),bpgrid);
RGARCH.psi0Kern=ksdensity(RGARCH.theta_dist(:,6),psi0grid);
RGARCH.psi1Kern=ksdensity(RGARCH.theta_dist(:,7),psi1grid);
RGARCH.sigbvKern=ksdensity(RGARCH.theta_dist(:,8),sigbvgrid);
RGARCH.omgvKern=ksdensity(RGARCH.theta_dist(:,9),omgvgrid);
RGARCH.rhoKern=ksdensity(RGARCH.theta_dist(:,10),rhogrid);
RGARCH.sigvKern=ksdensity(RGARCH.theta_dist(:,11),sigvgrid);
RGARCH.alphaKern=ksdensity(RGARCH.theta_dist(:,12),alphagrid);

save('predictive_results.mat');

%%
figure;
subplot(4,3,1); plot(psi0grid,[GARCHN.psi0Kern',GARCHT.psi0Kern',TGARCHT.psi0Kern',RGARCH.psi0Kern']);
title('ABC Posterior - \psi_0'); 
legend('GARCH-Normal', 'GARCH-T', 'TGARCH-T','RealizedGARCH','Location','northwest');
subplot(4,3,2); plot(psi1grid,[GARCHN.psi1Kern',GARCHT.psi1Kern',TGARCHT.psi1Kern',RGARCH.psi1Kern']);
title('ABC Posterior - \psi_1');
subplot(4,3,3); plot(sigbvgrid,[GARCHN.sigbvKern',GARCHT.sigbvKern',TGARCHT.sigbvKern',RGARCH.sigbvKern']);
title('ABC Posterior - \sigma_{BV}'); 
subplot(4,3,4); plot(omgvgrid,[GARCHN.omgvKern',GARCHT.omgvKern',TGARCHT.omgvKern',RGARCH.omgvKern']);
title('ABC Posterior - \omega'); 
subplot(4,3,5); plot(rhogrid,[GARCHN.rhoKern',GARCHT.rhoKern',TGARCHT.rhoKern',RGARCH.rhoKern']);
title('ABC Posterior - \rho'); 
subplot(4,3,6); plot(sigvgrid,[GARCHN.sigvKern',GARCHT.sigvKern',TGARCHT.sigvKern',RGARCH.sigvKern']);
title('ABC Posterior - \sigma_h'); 
subplot(4,3,7); plot(alphagrid,[GARCHN.alphaKern',GARCHT.alphaKern',TGARCHT.alphaKern',RGARCH.alphaKern']);
title('ABC Posterior - \alpha'); 
subplot(4,3,8); plot(del0grid,[GARCHN.del0Kern',GARCHT.del0Kern',TGARCHT.del0Kern',RGARCH.del0Kern']);
title('ABC Posterior - \delta^0'); 
subplot(4,3,9); plot(bpgrid,[GARCHN.bpKern',GARCHT.bpKern',TGARCHT.bpKern',RGARCH.bpKern']);
title('ABC Posterior - \beta'); 
subplot(4,3,10); plot(apgrid,[GARCHN.apKern',GARCHT.apKern',TGARCHT.apKern',RGARCH.apKern']);
title('ABC Posterior - \gamma'); 
subplot(4,3,11); plot(mupgrid,[GARCHN.mupKern',GARCHT.mupKern',TGARCHT.mupKern',RGARCH.mupKern']);
title('ABC Posterior - \mu'); 
subplot(4,3,12); plot(sigpgrid,[GARCHN.sigpKern',GARCHT.sigpKern',TGARCHT.sigpKern',RGARCH.sigpKern']);
title('ABC Posterior - \sigma_z'); 

