%% iteratively draw the data over a loop

clear; clc;
addpath('MISC_fn');
addpath('MCMCRoutines');

seed=654321; %fix seed so that first data set is same as the single set example
rng(seed);

%number of random sampling iterations
K=100;
T=5000; 
theta1=0.8; theta2=0.6; sig=1;

%number of ABC draws per run
eps=50/(T.^(3/2));
P=500;
reps=round(P/eps);

%storage vectors for the draws of parameters and predictives for time T+1
dist1_draws=zeros(P,K*3);
pred1_draws=zeros(P,K);
dist2_draws=zeros(P,K*3);
pred2_draws=zeros(P,K);
dist3_draws=zeros(P,K*3);
pred3_draws=zeros(P,K);
dist4_draws=zeros(P,K*3);
pred4_draws=zeros(P,K);
MCMC_draws=zeros(P,K*3);
predMCMC_draws=zeros(P,K);

%grids for densities
lt=0; ut=0.99; %draw from prior theta~unif(0,0.99)
ls=0.1; us=3.0;
    
grid=100;
thetag=linspace(lt,ut,grid);
sigg=linspace(ls,us,grid);
yllim=-10; yulim=10;
ygrid=linspace(yllim,yulim,grid);

%storage for the kernel density estimates of posteriors and predictives
t1_post_kern=zeros(grid,K);
t2_post_kern=zeros(grid,K);
s_post_kern=zeros(grid,K);
pred_kern_MCMC=zeros(grid,K);

t1_kern_dist1=zeros(grid,K);
t2_kern_dist1=zeros(grid,K);
s_kern_dist1=zeros(grid,K);
pred1_kern=zeros(grid,K);

t1_kern_dist2=zeros(grid,K);
t2_kern_dist2=zeros(grid,K);
s_kern_dist2=zeros(grid,K);
pred2_kern=zeros(grid,K);

t1_kern_dist3=zeros(grid,K);
t2_kern_dist3=zeros(grid,K);
s_kern_dist3=zeros(grid,K);
pred3_kern=zeros(grid,K);

t1_kern_dist4=zeros(grid,K);
t2_kern_dist4=zeros(grid,K);
s_kern_dist4=zeros(grid,K);
pred4_kern=zeros(grid,K);

%4 sets of statistics for each of the three parameter plus the predictive 
RMSE_theta_pdf=zeros(4,4,K); 
RMSE_theta_cdf=zeros(4,4,K); 
Hellinger_theta_pdf=zeros(4,4,K); 
Hellinger_theta_cdf=zeros(4,4,K); 
KS_stats=zeros(4,8,K); %saves the KS stats as well as p-vals
KS_reject=zeros(4,4,K); %count the number of times KS is rejected at 1% alpha
alpha=0.01;

%predictive socres
logscores=zeros(K,5);
qscores=zeros(K,5);
crpscores=zeros(K,5);

%integration of overlapping densities - applicable to the posterior of
%parameters as well
overlap=zeros(4,4,K);
overlapP=zeros(4,4,K);

for k=1:K
    rng(seed+k); %seed reset to ensure expanding samples in y
    %simulate the data
    %generate T+1 points for computing predictive scores
    [y]=simMA2(T+1,theta1,theta2,sig);
    yout=y(end);
    y=y(1:T);

    %compute the autocovariances of the data up to lag 4
    eta_y=autocorr(y,4);
    eta_y(1)=var(y); %replace the first element by the variance

    t1_draw=lt+rand(reps,1)*(ut-lt);
    t2_draw=lt+rand(reps,1)*(ut-lt);
    s_draw=ls+rand(reps,1)*(us-ls);

    %storage vectors for the summary statistics of z
    eta_z=zeros(reps,5); %store all for processing later

    parfor i=1:reps %ABC loop
       z=simMA2(T,t1_draw(i),t2_draw(i),s_draw(i));
       acf=autocorr(z,4);
       acf(1)=var(z);
       eta_z(i,:)=acf';
    end

    %computing the distance based on summary stats
    [dist1]= sstatDist_Euclidean(eta_z(:,1:2), eta_y(1:2)', reps);
    [dist2]= sstatDist_Euclidean(eta_z(:,1:3), eta_y(1:3)', reps);
    [dist3]= sstatDist_Euclidean(eta_z(:,1:4), eta_y(1:4)', reps);
    [dist4]= sstatDist_Euclidean(eta_z(:,1:5), eta_y(1:5)', reps);

    %sorting the draws based on various statistics
    theta_dist1=[t1_draw,t2_draw,s_draw,dist1];
    theta_dist1_sorted=sortrows(theta_dist1,4);

    theta_dist2=[t1_draw,t2_draw,s_draw,dist2];
    theta_dist2_sorted=sortrows(theta_dist2,4);

    theta_dist3=[t1_draw,t2_draw,s_draw,dist3];
    theta_dist3_sorted=sortrows(theta_dist3,4);

    theta_dist4=[t1_draw,t2_draw,s_draw,dist4];
    theta_dist4_sorted=sortrows(theta_dist4,4);

    %select the chosen draws;
    theta_dist1=theta_dist1_sorted(1:P,1:3);
    dist1_draws(:,3*(k-1)+1:3*k)=theta_dist1;
    [t1_kern_dist1(:,k)]=Gael_kernel(thetag,theta_dist1(:,1));
    [t2_kern_dist1(:,k)]=Gael_kernel(thetag,theta_dist1(:,2));
    [s_kern_dist1(:,k)]=Gael_kernel(sigg,theta_dist1(:,3));

    theta_dist2=theta_dist2_sorted(1:P,1:3);
    dist2_draws(:,3*(k-1)+1:3*k)=theta_dist2;
    [t1_kern_dist2(:,k)]=Gael_kernel(thetag,theta_dist2(:,1));
    [t2_kern_dist2(:,k)]=Gael_kernel(thetag,theta_dist2(:,2));
    [s_kern_dist2(:,k)]=Gael_kernel(sigg,theta_dist2(:,3));

    theta_dist3=theta_dist3_sorted(1:P,1:3);
    dist3_draws(:,3*(k-1)+1:3*k)=theta_dist3;    
    [t1_kern_dist3(:,k)]=Gael_kernel(thetag,theta_dist3(:,1));
    [t2_kern_dist3(:,k)]=Gael_kernel(thetag,theta_dist3(:,2));
    [s_kern_dist3(:,k)]=Gael_kernel(sigg,theta_dist3(:,3));

    theta_dist4=theta_dist4_sorted(1:P,1:3);
    dist4_draws(:,3*(k-1)+1:3*k)=theta_dist4;
    [t1_kern_dist4(:,k)]=Gael_kernel(thetag,theta_dist4(:,1));
    [t2_kern_dist4(:,k)]=Gael_kernel(thetag,theta_dist4(:,2));
    [s_kern_dist4(:,k)]=Gael_kernel(sigg,theta_dist4(:,3));

    % using the ABC draws to obtain predictive density of obs T+1
    eps_t=normrnd(zeros(1,P),ones(1,P));

    %order 1
    [resids] = resids_MA2(y,theta_dist1(:,1)',theta_dist1(:,2)');
    pred1=theta_dist1(:,3)'.*eps_t+theta_dist1(:,1)'.*resids(T,:)+theta_dist1(:,2)'.*resids(T-1,:);
    pred1_draws(:,k)=pred1;
    pred1_kern(:,k)=Gael_kernel(ygrid,pred1);

    %order 2
    [resids] = resids_MA2(y,theta_dist2(:,1)',theta_dist2(:,2)');
    pred2=theta_dist2(:,3)'.*eps_t+theta_dist2(:,1)'.*resids(T,:)+theta_dist2(:,2)'.*resids(T-1,:);
    pred2_draws(:,k)=pred2;
    pred2_kern(:,k)=Gael_kernel(ygrid,pred2);

    %order 3
    [resids] = resids_MA2(y,theta_dist3(:,1)',theta_dist3(:,2)');
    pred3=theta_dist3(:,3)'.*eps_t+theta_dist3(:,1)'.*resids(T,:)+theta_dist3(:,2)'.*resids(T-1,:);
    pred3_draws(:,k)=pred3;
    pred3_kern(:,k)=Gael_kernel(ygrid,pred3);

    %order 4
    [resids] = resids_MA2(y,theta_dist4(:,1)',theta_dist4(:,2)');
    pred4=theta_dist4(:,3)'.*eps_t+theta_dist4(:,1)'.*resids(T,:)+theta_dist4(:,2)'.*resids(T-1,:);
    pred4_draws(:,k)=pred4;
    pred4_kern(:,k)=Gael_kernel(ygrid,pred4);


    %% Run MCMC for comparison
    [pred_MCMC,store_theta,store_sig,t1_post_kern(:,k),t2_post_kern(:,k),s_post_kern(:,k),pred_kern_MCMC(:,k)]=MA2_fullMCMC_func(y,theta1,theta2,sig);

    %% post-processing - computing RMSE of the densities
    RMSE_theta_pdf(1,:,k)=mean([t1_kern_dist1(:,k)-t1_post_kern(:,k),t1_kern_dist2(:,k)-t1_post_kern(:,k),t1_kern_dist3(:,k)-t1_post_kern(:,k),t1_kern_dist4(:,k)-t1_post_kern(:,k)].^2);
    RMSE_theta_pdf(2,:,k)=mean([t2_kern_dist1(:,k)-t2_post_kern(:,k),t2_kern_dist2(:,k)-t2_post_kern(:,k),t2_kern_dist3(:,k)-t2_post_kern(:,k),t2_kern_dist4(:,k)-t2_post_kern(:,k)].^2);
    RMSE_theta_pdf(3,:,k)=mean([s_kern_dist1(:,k)-s_post_kern(:,k),s_kern_dist2(:,k)-s_post_kern(:,k),s_kern_dist3(:,k)-s_post_kern(:,k),s_kern_dist4(:,k)-s_post_kern(:,k)].^2);
    RMSE_theta_pdf(4,:,k)=mean([pred1_kern(:,k)-pred_kern_MCMC(:,k),pred2_kern(:,k)-pred_kern_MCMC(:,k),pred3_kern(:,k)-pred_kern_MCMC(:,k),pred4_kern(:,k)-pred_kern_MCMC(:,k)].^2);
   
    Hellinger_theta_pdf(1,:,k)=mean([sqrt(t1_kern_dist1(:,k))-sqrt(t1_post_kern(:,k)),sqrt(t1_kern_dist2(:,k))-sqrt(t1_post_kern(:,k)),sqrt(t1_kern_dist3(:,k))-sqrt(t1_post_kern(:,k)),sqrt(t1_kern_dist4(:,k))-sqrt(t1_post_kern(:,k))].^2);
    Hellinger_theta_pdf(2,:,k)=mean([sqrt(t2_kern_dist1(:,k))-sqrt(t2_post_kern(:,k)),sqrt(t2_kern_dist2(:,k))-sqrt(t2_post_kern(:,k)),sqrt(t2_kern_dist3(:,k))-sqrt(t2_post_kern(:,k)),sqrt(t2_kern_dist4(:,k))-sqrt(t2_post_kern(:,k))].^2);
    Hellinger_theta_pdf(3,:,k)=mean([sqrt(s_kern_dist1(:,k))-sqrt(s_post_kern(:,k)),sqrt(s_kern_dist2(:,k))-sqrt(s_post_kern(:,k)),sqrt(s_kern_dist3(:,k))-sqrt(s_post_kern(:,k)),sqrt(s_kern_dist4(:,k))-sqrt(s_post_kern(:,k))].^2);
    Hellinger_theta_pdf(4,:,k)=mean([sqrt(pred1_kern(:,k))-sqrt(pred_kern_MCMC(:,k)),sqrt(pred2_kern(:,k))-sqrt(pred_kern_MCMC(:,k)),sqrt(pred3_kern(:,k))-sqrt(pred_kern_MCMC(:,k)),sqrt(pred4_kern(:,k))-sqrt(pred_kern_MCMC(:,k))].^2);

    t1_kern_cdf1=cumtrapz(thetag,t1_kern_dist1(:,k));
    t1_kern_cdf1=t1_kern_cdf1/t1_kern_cdf1(end);
    t1_kern_cdf2=cumtrapz(thetag,t1_kern_dist2(:,k));
    t1_kern_cdf2=t1_kern_cdf2/t1_kern_cdf2(end);
    t1_kern_cdf3=cumtrapz(thetag,t1_kern_dist3(:,k));
    t1_kern_cdf3=t1_kern_cdf3/t1_kern_cdf3(end);
    t1_kern_cdf4=cumtrapz(thetag,t1_kern_dist4(:,k));
    t1_kern_cdf4=t1_kern_cdf4/t1_kern_cdf4(end);
    t1_post_cdf=cumtrapz(thetag,t1_post_kern(:,k));
    t1_post_cdf=t1_post_cdf/t1_post_cdf(end);

    t2_kern_cdf1=cumtrapz(thetag,t2_kern_dist1(:,k));
    t2_kern_cdf1=t2_kern_cdf1/t2_kern_cdf1(end);
    t2_kern_cdf2=cumtrapz(thetag,t2_kern_dist2(:,k));
    t2_kern_cdf2=t2_kern_cdf2/t2_kern_cdf2(end);
    t2_kern_cdf3=cumtrapz(thetag,t2_kern_dist3(:,k));
    t2_kern_cdf3=t2_kern_cdf3/t2_kern_cdf3(end);
    t2_kern_cdf4=cumtrapz(thetag,t2_kern_dist4(:,k));
    t2_kern_cdf4=t2_kern_cdf4/t2_kern_cdf4(end);
    t2_post_cdf=cumtrapz(thetag,t2_post_kern(:,k));
    t2_post_cdf=t2_post_cdf/t2_post_cdf(end);

    s_kern_cdf1=cumtrapz(sigg,s_kern_dist1(:,k));
    s_kern_cdf1=s_kern_cdf1/s_kern_cdf1(end);
    s_kern_cdf2=cumtrapz(sigg,s_kern_dist2(:,k));
    s_kern_cdf2=s_kern_cdf2/s_kern_cdf2(end);
    s_kern_cdf3=cumtrapz(sigg,s_kern_dist3(:,k));
    s_kern_cdf3=s_kern_cdf3/s_kern_cdf3(end);
    s_kern_cdf4=cumtrapz(sigg,s_kern_dist4(:,k));
    s_kern_cdf4=s_kern_cdf4/s_kern_cdf4(end);
    s_post_cdf=cumtrapz(sigg,s_post_kern(:,k));
    s_post_cdf=s_post_cdf/s_post_cdf(end);

    pred_kern_cdf1=cumtrapz(ygrid,pred1_kern(:,k));
    pred_kern_cdf1=pred_kern_cdf1/pred_kern_cdf1(end);
    pred_kern_cdf2=cumtrapz(ygrid,pred2_kern(:,k));
    pred_kern_cdf2=pred_kern_cdf2/pred_kern_cdf2(end);
    pred_kern_cdf3=cumtrapz(ygrid,pred3_kern(:,k));
    pred_kern_cdf3=pred_kern_cdf3/pred_kern_cdf3(end);
    pred_kern_cdf4=cumtrapz(ygrid,pred4_kern(:,k));
    pred_kern_cdf4=pred_kern_cdf4/pred_kern_cdf4(end);
    pred_MCMC_cdf=cumtrapz(ygrid,pred_kern_MCMC(:,k));
    pred_MCMC_cdf=pred_MCMC_cdf/pred_MCMC_cdf(end);

    RMSE_theta_cdf(1,:,k)=mean([t1_kern_cdf1-t1_post_cdf,t1_kern_cdf2-t1_post_cdf,t1_kern_cdf3-t1_post_cdf,t1_kern_cdf4-t1_post_cdf].^2);
    RMSE_theta_cdf(2,:,k)=mean([t2_kern_cdf1-t2_post_cdf,t2_kern_cdf2-t2_post_cdf,t2_kern_cdf3-t2_post_cdf,t2_kern_cdf4-t2_post_cdf].^2);
    RMSE_theta_cdf(3,:,k)=mean([s_kern_cdf1-s_post_cdf,s_kern_cdf2-s_post_cdf,s_kern_cdf3-s_post_cdf,s_kern_cdf4-s_post_cdf].^2);
    RMSE_theta_cdf(4,:,k)=mean([pred_kern_cdf1-pred_MCMC_cdf,pred_kern_cdf2-pred_MCMC_cdf,pred_kern_cdf3-pred_MCMC_cdf,pred_kern_cdf4-pred_MCMC_cdf].^2);
    
    Hellinger_theta_cdf(1,:,k)=mean([sqrt(t1_kern_cdf1)-sqrt(t1_post_cdf),sqrt(t1_kern_cdf2)-sqrt(t1_post_cdf),sqrt(t1_kern_cdf3)-sqrt(t1_post_cdf),sqrt(t1_kern_cdf4)-sqrt(t1_post_cdf)].^2);
    Hellinger_theta_cdf(2,:,k)=mean([sqrt(t2_kern_cdf1)-sqrt(t2_post_cdf),sqrt(t2_kern_cdf2)-sqrt(t2_post_cdf),sqrt(t2_kern_cdf3)-sqrt(t2_post_cdf),sqrt(t2_kern_cdf4)-sqrt(t2_post_cdf)].^2);
    Hellinger_theta_cdf(3,:,k)=mean([sqrt(s_kern_cdf1)-sqrt(s_post_cdf),sqrt(s_kern_cdf2)-sqrt(s_post_cdf),sqrt(s_kern_cdf3)-sqrt(s_post_cdf),sqrt(s_kern_cdf4)-sqrt(s_post_cdf)].^2);
    Hellinger_theta_cdf(4,:,k)=mean([sqrt(pred_kern_cdf1)-sqrt(pred_MCMC_cdf),sqrt(pred_kern_cdf2)-sqrt(pred_MCMC_cdf),sqrt(pred_kern_cdf3)-sqrt(pred_MCMC_cdf),sqrt(pred_kern_cdf4)-sqrt(pred_MCMC_cdf)].^2);

    %% compute predictive scores
    [logscores(k,1), qscores(k,1), crpscores(k,1)]=calcPredScoresNum(ygrid,pred1_kern(:,k),pred_kern_cdf1,yout,yllim,yulim);
    [logscores(k,2), qscores(k,2), crpscores(k,2)]=calcPredScoresNum(ygrid,pred2_kern(:,k),pred_kern_cdf2,yout,yllim,yulim);
    [logscores(k,3), qscores(k,3), crpscores(k,3)]=calcPredScoresNum(ygrid,pred3_kern(:,k),pred_kern_cdf3,yout,yllim,yulim);
    [logscores(k,4), qscores(k,4), crpscores(k,4)]=calcPredScoresNum(ygrid,pred4_kern(:,k),pred_kern_cdf4,yout,yllim,yulim);
    [logscores(k,5), qscores(k,5), crpscores(k,5)]=calcPredScoresNum(ygrid,pred_kern_MCMC(:,k),pred_MCMC_cdf,yout,yllim,yulim);
    
    %% compute overlapping measures - including probabilistic one
    [overlap(1,1,k),overlapP(1,1,k)]=calcOVL(thetag,t1_kern_dist1(:,k),t1_post_kern(:,k),lt,ut);
    [overlap(1,2,k),overlapP(1,2,k)]=calcOVL(thetag,t1_kern_dist2(:,k),t1_post_kern(:,k),lt,ut);
    [overlap(1,3,k),overlapP(1,3,k)]=calcOVL(thetag,t1_kern_dist3(:,k),t1_post_kern(:,k),lt,ut);
    [overlap(1,4,k),overlapP(1,4,k)]=calcOVL(thetag,t1_kern_dist4(:,k),t1_post_kern(:,k),lt,ut);
    
    [overlap(2,1,k),overlapP(2,1,k)]=calcOVL(thetag,t2_kern_dist1(:,k),t2_post_kern(:,k),lt,ut);
    [overlap(2,2,k),overlapP(2,2,k)]=calcOVL(thetag,t2_kern_dist2(:,k),t2_post_kern(:,k),lt,ut);
    [overlap(2,3,k),overlapP(2,3,k)]=calcOVL(thetag,t2_kern_dist3(:,k),t2_post_kern(:,k),lt,ut);
    [overlap(2,4,k),overlapP(2,4,k)]=calcOVL(thetag,t2_kern_dist4(:,k),t2_post_kern(:,k),lt,ut);
    
    [overlap(3,1,k),overlapP(3,1,k)]=calcOVL(sigg,s_kern_dist1(:,k),s_post_kern(:,k),ls,us);
    [overlap(3,2,k),overlapP(3,2,k)]=calcOVL(sigg,s_kern_dist2(:,k),s_post_kern(:,k),ls,us);
    [overlap(3,3,k),overlapP(3,3,k)]=calcOVL(sigg,s_kern_dist3(:,k),s_post_kern(:,k),ls,us);
    [overlap(3,4,k),overlapP(3,4,k)]=calcOVL(sigg,s_kern_dist4(:,k),s_post_kern(:,k),ls,us);
    
    [overlap(4,1,k),overlapP(4,1,k)]=calcOVL(ygrid,pred1_kern(:,k),pred_kern_MCMC(:,k),yllim,yulim);
    [overlap(4,2,k),overlapP(4,2,k)]=calcOVL(ygrid,pred2_kern(:,k),pred_kern_MCMC(:,k),yllim,yulim);
    [overlap(4,3,k),overlapP(4,3,k)]=calcOVL(ygrid,pred3_kern(:,k),pred_kern_MCMC(:,k),yllim,yulim);
    [overlap(4,4,k),overlapP(4,4,k)]=calcOVL(ygrid,pred4_kern(:,k),pred_kern_MCMC(:,k),yllim,yulim);

    %% KS test on densities
    %to filter draws from MCMC
    tmp=mod(1:length(store_sig),40);
    take=find(tmp==0);
    
    MCMC_draws(:,3*(k-1)+1:3*k)=[store_theta(take,:),store_sig(take)];
    predMCMC_draws(:,k)=pred_MCMC';
    
    [KS_reject(1,1,k),KS_stats(1,2,k),KS_stats(1,1,k)]=kstest2(theta_dist1(:,1),store_theta(take,1),'Alpha',alpha);
    [KS_reject(1,2,k),KS_stats(1,4,k),KS_stats(1,3,k)]=kstest2(theta_dist2(:,1),store_theta(take,1),'Alpha',alpha);
    [KS_reject(1,3,k),KS_stats(1,6,k),KS_stats(1,5,k)]=kstest2(theta_dist3(:,1),store_theta(take,1),'Alpha',alpha);
    [KS_reject(1,4,k),KS_stats(1,8,k),KS_stats(1,7,k)]=kstest2(theta_dist4(:,1),store_theta(take,1),'Alpha',alpha);

    [KS_reject(2,1,k),KS_stats(2,2,k),KS_stats(2,1,k)]=kstest2(theta_dist1(:,2),store_theta(take,2),'Alpha',alpha);
    [KS_reject(2,2,k),KS_stats(2,4,k),KS_stats(2,3,k)]=kstest2(theta_dist2(:,2),store_theta(take,2),'Alpha',alpha);
    [KS_reject(2,3,k),KS_stats(2,6,k),KS_stats(2,5,k)]=kstest2(theta_dist3(:,2),store_theta(take,2),'Alpha',alpha);
    [KS_reject(2,4,k),KS_stats(2,8,k),KS_stats(2,7,k)]=kstest2(theta_dist4(:,2),store_theta(take,2),'Alpha',alpha);

    [KS_reject(3,1,k),KS_stats(3,2,k),KS_stats(3,1,k)]=kstest2(theta_dist1(:,3),store_sig(take),'Alpha',alpha);
    [KS_reject(3,2,k),KS_stats(3,4,k),KS_stats(3,3,k)]=kstest2(theta_dist2(:,3),store_sig(take),'Alpha',alpha);
    [KS_reject(3,3,k),KS_stats(3,6,k),KS_stats(3,5,k)]=kstest2(theta_dist3(:,3),store_sig(take),'Alpha',alpha);
    [KS_reject(3,4,k),KS_stats(3,8,k),KS_stats(3,7,k)]=kstest2(theta_dist4(:,3),store_sig(take),'Alpha',alpha);

    [KS_reject(4,1,k),KS_stats(4,2,k),KS_stats(4,1,k)]=kstest2(pred1,pred_MCMC','Alpha',alpha);
    [KS_reject(4,2,k),KS_stats(4,4,k),KS_stats(4,3,k)]=kstest2(pred2,pred_MCMC','Alpha',alpha);
    [KS_reject(4,3,k),KS_stats(4,6,k),KS_stats(4,5,k)]=kstest2(pred3,pred_MCMC','Alpha',alpha);
    [KS_reject(4,4,k),KS_stats(4,8,k),KS_stats(4,7,k)]=kstest2(pred4,pred_MCMC','Alpha',alpha);
end

%average the RMSE over the draws
RMSE_pdf_avg=mean(RMSE_theta_pdf,3);
RMSE_cdf_avg=mean(RMSE_theta_cdf,3);

%average the Hellinger over the draws
Hellinger_pdf_avg=mean(Hellinger_theta_pdf,3);
Hellinger_cdf_avg=mean(Hellinger_theta_cdf,3);

%proportion of times KS is rejected
KS_reject_p=mean(KS_reject,3);

%average of the KS stat (supremum of the CDF differences)
KS_avg=mean(KS_stats,3);

%average of predictive scores
logscores_avg=mean(logscores);
qscores_avg=mean(qscores);
crpscores_avg=mean(crpscores);

%average of OVL
OVL_avg=mean(overlap,3);
OVLP_avg=mean(overlapP,3);

save('MA2_ABC_T5000_100reps.mat');