clear; clc;
addpath('MISC_fn');
addpath('MCMCRoutines');
load('SVdata.mat');
seed=654321;
rng(seed);

T=1000;
y=y(1:T);

%number of MCMC reps
reps=20000;
burnin=5000;
g=reps+burnin;

%upper and lower bounds for k and sig prior
% lk=0.5; uk=0.99; %for two parameter case
% ls=0.05; us=0.5; %for two parameter case
lk=0.7; uk=0.99; %for one parameter case
ls=0.05; us=0.3; %for one parameter case

%storage vectors
store_k=zeros(reps,1);
store_sigv=zeros(reps,1);
store_lastH=zeros(reps,1);

%initialize vales
curr_k=k;
curr_sigv=sigv;
curr_h=zeros(T,1);
prev_h=curr_h;
tic;
for i=1:g
    %draw the latent state - use the sparse matrix with 7-point approx.
    [curr_h]=draw_SV_h(y,d,curr_k,curr_sigv,prev_h);
    
    %draw the parameters given the state - both resulting in Gibbs
    curr_k=draw_SV_k(curr_h,d,curr_sigv,lk,uk);
    %fix k
%     curr_k=k;
    
    curr_sigv=draw_SV_sigv(curr_h,d,curr_k,ls,us);
    %fix sigv
%     curr_sigv=sigv;
    
    if(i>burnin)
       store_k(i-burnin)=curr_k;
       store_sigv(i-burnin)=curr_sigv;
       store_lastH(i-burnin)=curr_h(end);
    end
    prev_h=curr_h; %need this for MH of the state
    if(mod(i,100)==0)
        disp([' MCMC iteration ' num2str(i) ', k=' num2str(curr_k) ', sigv=' num2str(curr_sigv) ]);
    end
end

%trace plots
figure;
subplot(2,2,1); plot(store_k); title('Trace plot of k');
subplot(2,2,2); plot(store_sigv); title('Trace plot of \sigma_v');
subplot(2,2,3); plot(store_lastH); title('Trace plot of h_T');
subplot(2,2,4); plot(curr_h); title('Final draw of log volatility');

%% extracting the posterior predictive distribution for y_T+1
%using thinned out draws - take every 20th draw 
tmp=mod(1:reps,20);
take=find(tmp==0);
ygrid=linspace(-10,10,100);

predy_MCMC=predY_exact_SVOU(ygrid,d,store_k(take),store_sigv(take),store_lastH(take));

mcmc_time=toc; 
figure;
plot(ygrid,predy_MCMC); title('MCMC based predictive density for y_{T+1}');

kg=linspace(lk,uk,100);
[k_post_kern]=Gael_kernel(kg,store_k(take));
sigg=linspace(ls,us,100);
[s_post_kern]=Gael_kernel(sigg,store_sigv(take));
figure;
subplot(2,1,1); plot(kg,k_post_kern); title('Posterior of k');
subplot(2,1,2); plot(sigg,s_post_kern); title('Posterior of \sigma_v');
save('SV_MCMC_k_sig.mat');