function [ predy ] = predY_BPF_SVOU( ygrid, y, d, k, sig )
%Bootstrap particle filter (BPF) for the computation of the likelihood of
%the SV model.
%Model: yt=exp(xt/2)*et
%xt+1=c+b*(xt)+csig*vt
%d is the unconditional mean of xt

%define the parameters of the SV process
marg_mu=d;
marg_sig=sqrt((sig^2)/(1-k^2));
c=d*(1-k);
b=k;
csig=sig;
% seed=123456;
% randg(seed);

%set up the relevant matrices
T=length(y);
N=5000;%number of particles
filtered_mean_BPF=zeros(T,1);
% loglike_BPF=zeros(T,1);
filtered_particle=zeros(T,N);

%go through the filtering for t=1
new_particle=normrnd(marg_mu,marg_sig,1,N); 
%compute the particle weights (log measurement density given particles)
incr_weight=-0.5*log(2*pi)-0.5.*new_particle-0.5.*(y(1).^2)./exp(new_particle);
lw=incr_weight-log(N);
[norm_weight,~]=normLogWeights(lw);
%compute filtered mean (as a check)
filtered_mean_BPF(1)=sum(norm_weight.*new_particle);

%compute the predictive likelihood
% like_BPF(1)=sum(weight);
% loglike_BPF(1)=lse;
% pred_like=sum(weight);

%resample the particles
filtered_particle(1,:)=randsample(new_particle,N,true,norm_weight)';

%repeat the loop for t=2:T
for t=2:T
    %sample new particles for time t
    mux=c+b.*filtered_particle(t-1,:);
    new_particle=normrnd(mux,csig*ones(1,N));

    %compute the particle weights
    incr_weight=-0.5*log(2*pi)-0.5.*new_particle-0.5*(y(t).^2)./exp(new_particle);
    lw=incr_weight-log(N);
    [norm_weight,~]=normLogWeights(lw);
    %compute filtered mean (as a check)
    filtered_mean_BPF(t)=sum(norm_weight.*new_particle);
    %compute the predictive likelihood for each t  - not needed here
%     loglike_BPF(t)=lse; %not needed for this case
    
    %resample the particles
    filtered_particle(t,:)=randsample(new_particle,N,true,norm_weight)';
end

%one step ahead predictive particle
mux=c+b.*filtered_particle(T,:);
new_particle=normrnd(mux,csig*ones(1,N));
predy=mean(exp(lnormpdf(repmat(ygrid',1,N),zeros(length(ygrid),N),repmat(exp(new_particle./2),length(ygrid),1))),2);
end

